#ifndef GAMEDATA_HPP
#define GAMEDATA_HPP

#include "Game.hpp"

class GameData
{
public:
	enum
	{
		PLAYER_COUNT = Game::PLAYER_TYPE_MAX,
		LANGUAGE_ENGLISH = 0,
		LANGUAGE_JAPANESE,
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
		RESERVE_SIZE = 97,
#else
		RESERVE_SIZE = 100,
#endif
	};
	struct PlayerData
	{
		bool lock;
		int level;
		int hp;
		int attack;
		int dcy;
		int hp_up_rate;
		bool clear;
	};
	GameData(void);
	~GameData(void);
	void Initialize(void);
	bool Load(void);
	void Save(void);
	void AddCrystal(int add_count);
	int GetCrystal(void);
	void SetSoundVolume(int sound_volume);
	int GetSoundVolume(void);
	int LevelUpCrystal(int player_type);
	bool CheckLevelUp(int player_type);
	void LevelUp(int player_type);
	const PlayerData* GetPlayerData(int player_type);
	int UnlockCrystal(int player_type);
	bool CheckUnlock(int player_type);
	void Unlock(int player_type);
	void SetLanguage(int language);
	int GetLanguage(void);
	void SetClear(int player_type);
	bool GetClear(int player_type);
	bool CheckUnlockLevelLimit(void);
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
	void SetPlayMovieTime(unsigned long long play_movie_time);
	unsigned long long GetPlayMovieTime(void);
	void ResetMovieCountLeft(void);
	void DecrementMovieCountLeft(void);
	int GetMovieCountLeft(void);
	void SetControllerThreshold(int controller_threshold);
	int GetControllerThreshold(void);
#endif
private:
	GameData(GameData&);
	GameData& operator = (GameData&);
	struct Data
	{
		PlayerData player_data[PLAYER_COUNT];
		int crystal;
		int sound_volume; // 0-10
		int language;
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
		unsigned long long play_movie_time;
		int play_movie_count_left;
		int controller_threshold; // 10-200
#endif
		int reserve[RESERVE_SIZE];
		int check_sum;
	};
	Data data;
};

#endif
