#include "TinyLibrary.hpp"
#include "Global.hpp"
#include "Sound.hpp"
#include "Popup.hpp"

Popup::Popup(void)
:type(TYPE_SELECT)
,result(RESULT_NONE)
,select(RESULT_CANCEL)
,before_select(RESULT_CANCEL)
,popup_x(0)
,popup_y((Screen::HEIGHT - 38) / 2)
,before_button(1)
{
}

Popup::~Popup(void)
{
}

void Popup::Initialize(int type)
{
	this->type = type;
}

void Popup::Update(void)
{
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
	const ImageData* image_data = GetImageData(BUTTON_ON);
	float tap_x;
	float tap_y;
	GetTapPosition(tap_x, tap_y);
	float rate = Screen::WIDTH / dms::Graphics::GetWindowWidth();
	float touch_x = (tap_x + dms::Graphics::GetWindowWidth() * 0.5f) * rate;
	float touch_y = (-(tap_y - dms::Graphics::GetWindowHeight() * 0.5f) - GetBannerHeight()) * rate;
	float button_up_left = static_cast<float>(this->popup_x + 46);
	float button_up_top = static_cast<float>(this->popup_y + 16);
	float button_up_right = button_up_left + static_cast<float>(image_data->width);
	float button_up_bottom = button_up_top + static_cast<float>(image_data->height);
	float button_down_left = static_cast<float>(this->popup_x + 46);
	float button_down_top = static_cast<float>(this->popup_y + 25);
	float button_down_right = button_down_left + static_cast<float>(image_data->width);
	float button_down_bottom = button_down_top + static_cast<float>(image_data->height);
	bool button_up = false;
	bool button_down = false;
	if((button_up_left <= touch_x) && (touch_x <= button_up_right) && (button_up_top <= touch_y) && (touch_y <= button_up_bottom))
	{
		button_up = true;
	}
	if((button_down_left <= touch_x) && (touch_x <= button_down_right) && (button_down_top <= touch_y) && (touch_y <= button_down_bottom))
	{
		button_down = true;
	}
#endif
	Global& global = Global::GetInstance();
	Controller& controller = Controller::GetInstance();
	Sound& sound = Sound::GetInstance();
	unsigned int button = controller.GetButton();
	if(this->type == TYPE_SELECT)
	{
		if((this->before_button == 0) && (button & Controller::BUTTON_UP))
		{
			this->select = RESULT_DECISION;
		}
		else if((this->before_button == 0) && (button & Controller::BUTTON_DOWN))
		{
			this->select = RESULT_CANCEL;
		}
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
		if(button_up == true)
		{
			this->select = RESULT_DECISION;
		}
		else if(button_down == true)
		{
			this->select = RESULT_CANCEL;
		}
#endif
	}
	else
	{
		this->select = RESULT_DECISION;
	}
	if((this->before_button == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
	{
		sound.Play(SOUND_DECISION);
		this->result = this->select;
	}
	else
	{
		if(this->select != this->before_select)
		{
			if(this->type == TYPE_SELECT)
			{
				sound.Play(SOUND_CURSOR);
			}
		}
	}
	this->before_select = this->select;
	this->before_button = button;
	controller.Reset();
}

void Popup::DrawFrame(void)
{
	Screen& screen = Screen::GetInstance();
	const ImageData* image_data = GetImageData(WINDOW);
	screen.DrawSprite(image_data->buffer, this->popup_x, this->popup_y, image_data->width, image_data->height, 0);
	if(this->type == TYPE_SELECT)
	{
		if(this->select != RESULT_NONE)
		{
			Screen& screen = Screen::GetInstance();
			if(this->select == RESULT_DECISION)
			{
				image_data = GetImageData(BUTTON_ON);
				screen.DrawSprite(image_data->buffer, this->popup_x + 46, this->popup_y + 16, image_data->width, image_data->height, 0);
				image_data = GetImageData(BUTTON_OFF);
				screen.DrawSprite(image_data->buffer, this->popup_x + 46, this->popup_y + 25, image_data->width, image_data->height, 0);
			}
			else
			{
				image_data = GetImageData(BUTTON_OFF);
				screen.DrawSprite(image_data->buffer, this->popup_x + 46, this->popup_y + 16, image_data->width, image_data->height, 0);
				image_data = GetImageData(BUTTON_ON);
				screen.DrawSprite(image_data->buffer, this->popup_x + 46, this->popup_y + 25, image_data->width, image_data->height, 0);
			}
		}
	}
}

int Popup::GetResult(void)
{
	return this->result;
}
