#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
#include "../../Common.hpp"
#include "../SoundData.hpp"
#else
#include "SoundPlayer.hpp"
#endif
#include "Global.hpp"
#include "Sound.hpp"

Sound* Sound::sound;

Sound::Sound(void)
:before_bgm_sound_number(-1)
,playing_bgm(false)
{
	for(int i = 0; i < SOUND_MAX; ++ i)
	{
		sound_handle[i] = 0;
	}
}

Sound::~Sound(void)
{
	for(int i = 0; i < SOUND_MAX; ++ i)
	{
		this->sound_handle[i] = 0;
	}
}

void Sound::Initialize(void)
{
	Sound::sound = new Sound();
#if !defined(_WIN32) && !defined(__ANDROID__) && !defined(__APPLE__)
	SoundPlayer::Initialize(11025);
#endif
}

void Sound::Finalize(void)
{
#if !defined(_WIN32) && !defined(__ANDROID__) && !defined(__APPLE__)
	SoundPlayer::Finalize();
#endif
	if(Sound::sound != NULL)
	{
		delete Sound::sound;
		Sound::sound = NULL;
	}
}

Sound& Sound::GetInstance()
{
	return *Sound::sound;
}

void Sound::Load(void)
{
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
	Common& common = Common::GetInstance();
	dms::SoundPlayer* sound_player = common.resource->GetSoundPlayer(BGM_PLAYER);
#else
	SoundPlayer& sound_player = SoundPlayer::GetInstance();
	Global& global = Global::GetInstance();
	if(global.game_data.GetLanguage() == GameData::LANGUAGE_ENGLISH)
	{
		this->sound_handle[SOUND_BGM_TITLE] = sound_player.Append(0, "KemonoRogue/title.raw", true);
		this->sound_handle[SOUND_BGM_STAGE] = sound_player.Append(0, "KemonoRogue/stage.raw", true);
	}
	else
	{
		this->sound_handle[SOUND_BGM_TITLE] = sound_player.Append(0, "KemonoRogue_jp/title.raw", true);
		this->sound_handle[SOUND_BGM_STAGE] = sound_player.Append(0, "KemonoRogue_jp/stage.raw", true);
	}
	const SoundData* sound_data = GetSoundData(SOUND_CURSOR);
	this->sound_handle[SOUND_CURSOR] = sound_player.Append(1, sound_data->buffer, sound_data->size, false);
	sound_data = GetSoundData(SOUND_DECISION);
	this->sound_handle[SOUND_DECISION] = sound_player.Append(1, sound_data->buffer, sound_data->size, false);
	sound_data = GetSoundData(SOUND_DOWN);
	this->sound_handle[SOUND_DOWN] = sound_player.Append(1, sound_data->buffer, sound_data->size, false);
	sound_data = GetSoundData(SOUND_OPEN);
	this->sound_handle[SOUND_OPEN] = sound_player.Append(1, sound_data->buffer, sound_data->size, false);
	sound_data = GetSoundData(SOUND_HIT001);
	this->sound_handle[SOUND_HIT001] = sound_player.Append(1, sound_data->buffer, sound_data->size, false);
#endif
}

void Sound::Release(void)
{
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
	Common& common = Common::GetInstance();
	dms::SoundPlayer* sound_player = common.resource->GetSoundPlayer(BGM_PLAYER);
	if(sound_player != NULL)
	{
		sound_player->Stop();
	}
#else
	SoundPlayer& sound_player = SoundPlayer::GetInstance();
	sound_player.StopAll();
	sound_player.Delete(this->sound_handle[SOUND_BGM_TITLE]);
	sound_player.Delete(this->sound_handle[SOUND_BGM_STAGE]);
	sound_player.Delete(this->sound_handle[SOUND_CURSOR]);
	sound_player.Delete(this->sound_handle[SOUND_DECISION]);
	sound_player.Delete(this->sound_handle[SOUND_DOWN]);
	sound_player.Delete(this->sound_handle[SOUND_OPEN]);
	sound_player.Delete(this->sound_handle[SOUND_HIT001]);
#endif
}

void Sound::Play(int sound_number)
{
	Global& global = Global::GetInstance();
	if((sound_number >= BGM_START) && (sound_number <= BGM_END))
	{
		this->playing_bgm = true;
		before_bgm_sound_number = sound_number;
	}
	if(global.game_data.GetSoundVolume() == 0)
	{
		return;
	}
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
	Common& common = Common::GetInstance();
	dms::SoundPlayer* sound_player;
	switch(sound_number)
	{
	case SOUND_BGM_TITLE:
		sound_player = common.resource->GetSoundPlayer(BGM_PLAYER);
		sound_player->Play(BGM_TITLE);
		break;
	case SOUND_BGM_STAGE:
		sound_player = common.resource->GetSoundPlayer(BGM_PLAYER);
		sound_player->Play(BGM_STAGE);
		break;
	case SOUND_CURSOR:
		sound_player = common.resource->GetSoundPlayer(SE_PLAYER_COMMON);
		sound_player->Play(SE_COMMON_CURSOR);
		break;
	case SOUND_DECISION:
		sound_player = common.resource->GetSoundPlayer(SE_PLAYER_COMMON);
		sound_player->Play(SE_COMMON_DECISION);
		break;
	case SOUND_DOWN:
		sound_player = common.resource->GetSoundPlayer(SE_PLAYER_COMMON);
		sound_player->Play(SE_COMMON_DOWN);
		break;
	case SOUND_OPEN:
		sound_player = common.resource->GetSoundPlayer(SE_PLAYER_COMMON);
		sound_player->Play(SE_COMMON_OPEN);
		break;
	case SOUND_HIT001:
		sound_player = common.resource->GetSoundPlayer(SE_PLAYER_COMMON);
		sound_player->Play(SE_COMMON_HIT001);
		break;
	}
#else
	SoundPlayer& sound_player = SoundPlayer::GetInstance();
	sound_player.Play(this->sound_handle[sound_number]);
#endif
}

void Sound::StopAll(void)
{
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
	Common& common = Common::GetInstance();
	dms::SoundPlayer* sound_player;
	sound_player = common.resource->GetSoundPlayer(BGM_PLAYER);
	if(sound_player != NULL)
	{
		sound_player->Stop();
	}
	sound_player = common.resource->GetSoundPlayer(SE_PLAYER_COMMON);
	if(sound_player != NULL)
	{
		sound_player->Stop();
	}
#else
	SoundPlayer& sound_player = SoundPlayer::GetInstance();
	sound_player.StopAll();
#endif
	this->playing_bgm = false;
}

void Sound::Update(void)
{
#if !defined(_WIN32) && !defined(__ANDROID__) && !defined(__APPLE__)
	SoundPlayer& sound_player = SoundPlayer::GetInstance();
	sound_player.Update();
#endif
}

void Sound::Replay(void)
{
	if(this->before_bgm_sound_number == -1)
	{
		return;
	}
	Play(this->before_bgm_sound_number);
}

void Sound::ResetReplay(void)
{
	this->before_bgm_sound_number = -1;
}

bool Sound::BgmStatus(void)
{
	return this->playing_bgm;
}
